﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace mlang_console
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length < 1 || !File.Exists(args[0]))
            {
                Console.WriteLine("ファイルをコマンドライン引数に指定してください。");
                Console.ReadLine();
                return;
            }

            //テキストファイルを開く
            byte[] bs = System.IO.File.ReadAllBytes(args[0]);

            //文字コードを判別する
            System.Text.Encoding enc = DetectEncoding(bs);

            //結果を表示する
            if (enc == null)
            {
                Console.WriteLine("失敗しました。");
            }
            else
            {
                //エンコーディング名を表示する
                Console.WriteLine(enc.EncodingName);
            }

            Console.ReadLine();
        }

        /// <summary>
        /// IMultiLanguage2::DetectInputCodepageを使用して、文字コードを判別する
        /// </summary>
        /// <param name="bytes">文字コードを調べたいデータ。</param>
        /// <returns>第一候補のEncodingオブジェクト。
        /// 失敗した時は、null。</returns>
        public static System.Text.Encoding DetectEncoding(byte[] bytes)
        {
            //データが短い時は長くする
            if (bytes.Length < 256)
            {
                int loopCount = 256 / bytes.Length + 1;
                byte[] buf = new byte[loopCount * bytes.Length];
                for (int i = 0; i < loopCount; i++)
                {
                    Array.Copy(bytes, 0, buf, bytes.Length * i, bytes.Length);
                }
                bytes = buf;
            }

            //byte型配列をsbyte型配列に変換する
            sbyte[] sbyts = (sbyte[])(object)bytes;
            //sbyte[] sbyts = new sbyte[bs.Length];
            //System.Buffer.BlockCopy(bs, 0, sbyts, 0, bs.Length);

            //CMultiLanguageClassを作成
            MultiLanguage.IMultiLanguage2 ml = (MultiLanguage.IMultiLanguage2)
                new MultiLanguage.CMultiLanguageClass();
            if (ml == null)
            {
                return null;
            }

            //取得する候補の数だけ、tagDetectEncodingInfo配列を作成
            //ここでは、1つだけとする
            int scores = 1;
            MultiLanguage.tagDetectEncodingInfo[] detects =
                new MultiLanguage.tagDetectEncodingInfo[scores];

            System.Text.Encoding enc = null;
            int len = sbyts.Length;
            try
            {
                //文字コードを判別
                ml.DetectInputCodepage(
                    0, 0, ref sbyts[0], ref len, out detects[0], ref scores);
                //場合によっては（Visual Studio 2008以下？）、次のようになる
                //ml.DetectInputCodepage(
                //    0, 0, ref sbyts[0], ref len, ref detects[0], ref scores);
                //成功したか確認
                if (0 < scores)
                {
                    //一番初めのtagDetectEncodingInfoのEncodingを取得
                    //detectsはnDocPercentの大きい順番に並んでいる？
                    enc = System.Text.Encoding.GetEncoding((int)detects[0].nCodePage);
                }
            }
            finally
            {
                //後始末
                System.Runtime.InteropServices.Marshal.FinalReleaseComObject(ml);
            }

            return enc;
        }
    }
}
