﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports System.IO

Namespace mlang_console
    Class Program
        Shared Sub Main(args As String())
            If args.Length < 1 OrElse Not File.Exists(args(0)) Then
                Console.WriteLine("ファイルをコマンドライン引数に指定してください。")
                Console.ReadLine()
                Return
            End If

            'テキストファイルを開く
            Dim bs As Byte() = System.IO.File.ReadAllBytes(args(0))

            '文字コードを判別する
            Dim enc As System.Text.Encoding = DetectEncoding(bs)

            '結果を表示する
            If enc Is Nothing Then
                Console.WriteLine("失敗しました。")
            Else
                'エンコーディング名を表示する
                Console.WriteLine(enc.EncodingName)
            End If

            Console.ReadLine()
        End Sub

        ''' <summary>
        ''' IMultiLanguage2::DetectInputCodepageを使用して、文字コードを判別する
        ''' </summary>
        ''' <param name="bytes">文字コードを調べたいデータ。</param>
        ''' <returns>第一候補のEncodingオブジェクト。
        ''' 失敗した時は、null。</returns>
        Public Shared Function DetectEncoding(bytes As Byte()) As System.Text.Encoding
            'データが短い時は長くする
            If bytes.Length < 256 Then
                Dim loopCount As Integer = 256 \ bytes.Length + 1
                Dim buf As Byte() = New Byte(loopCount * bytes.Length - 1) {}
                For i As Integer = 0 To loopCount - 1
                    Array.Copy(bytes, 0, buf, bytes.Length * i, bytes.Length)
                Next
                bytes = buf
            End If

            'byte型配列をsbyte型配列に変換する
            Dim sbyts As SByte() = DirectCast(DirectCast(bytes, Object), SByte())
            'Dim sbyts(bytes.Length - 1) As System.SByte
            'System.Buffer.BlockCopy(bytes, 0, sbyts, 0, bytes.Length)

            'CMultiLanguageClassを作成
            Dim ml As MultiLanguage.IMultiLanguage2 = _
                DirectCast(New MultiLanguage.CMultiLanguageClass(),  _
                    MultiLanguage.IMultiLanguage2)
            If ml Is Nothing Then
                Return Nothing
            End If

            '取得する候補の数だけ、tagDetectEncodingInfo配列を作成
            'ここでは、1つだけとする
            Dim scores As Integer = 1
            Dim detects As MultiLanguage.tagDetectEncodingInfo() = _
                New MultiLanguage.tagDetectEncodingInfo(scores - 1) {}

            Dim enc As System.Text.Encoding = Nothing
            Dim len As Integer = sbyts.Length
            Try
                '文字コードを判別
                ml.DetectInputCodepage(0, 0, sbyts(0), len, detects(0), scores)
                '成功したか確認
                If 0 < scores Then
                    '一番初めのtagDetectEncodingInfoのEncodingを取得
                    'detectsはnDocPercentの大きい順番に並んでいる？
                    enc = System.Text.Encoding.GetEncoding(CInt(detects(0).nCodePage))
                End If
            Finally
                '後始末
                System.Runtime.InteropServices.Marshal.FinalReleaseComObject(ml)
            End Try

            Return enc
        End Function
    End Class
End Namespace
